@@ -27,11 +27,11 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 
 Portions of code from MODP_ASCII - Ascii transformations (upper/lower, etc)
-http://code.google.com/p/stringencoders/
+https://github.com/client9/stringencoders
 Copyright (c) 2007  Nick Galbreath -- nickg [at] modp [dot] com. All rights reserved.
 
 Numeric decoder derived from from TCL library
-http://www.opensource.apple.com/source/tcl/tcl-14/tcl/license.terms
+https://opensource.apple.com/source/tcl/tcl-14/tcl/license.terms
  * Copyright (c) 1988-1993 The Regents of the University of California.
  * Copyright (c) 1994 Sun Microsystems, Inc.
 */
@@ -61,87 +61,95 @@ tree doesn't have cyclic references.
 
 // Max decimals to encode double floating point numbers with
 #ifndef JSON_DOUBLE_MAX_DECIMALS
-#define JSON_DOUBLE_MAX_DECIMALS 15
+    #define JSON_DOUBLE_MAX_DECIMALS 15
 #endif
 
 // Max recursion depth, default for encoder
 #ifndef JSON_MAX_RECURSION_DEPTH
-#define JSON_MAX_RECURSION_DEPTH 1024
+    #define JSON_MAX_RECURSION_DEPTH 1024
 #endif
 
 // Max recursion depth, default for decoder
 #ifndef JSON_MAX_OBJECT_DEPTH
-#define JSON_MAX_OBJECT_DEPTH 1024
+    #define JSON_MAX_OBJECT_DEPTH 1024
 #endif
 
 /*
 Dictates and limits how much stack space for buffers UltraJSON will use before resorting to provided heap functions */
 #ifndef JSON_MAX_STACK_BUFFER_SIZE
-#define JSON_MAX_STACK_BUFFER_SIZE 131072
+    #define JSON_MAX_STACK_BUFFER_SIZE 1024
 #endif
 
 #ifdef _WIN32
 
-typedef __int64 JSINT64;
-typedef unsigned __int64 JSUINT64;
+    typedef __int64 JSINT64;
+    typedef unsigned __int64 JSUINT64;
 
-typedef __int32 JSINT32;
-typedef unsigned __int32 JSUINT32;
-typedef unsigned __int8 JSUINT8;
-typedef unsigned __int16 JSUTF16;
-typedef unsigned __int32 JSUTF32;
-typedef __int64 JSLONG;
+    typedef __int32 JSINT32;
+    typedef unsigned __int32 JSUINT32;
+    typedef unsigned __int8 JSUINT8;
+    typedef unsigned __int16 JSUTF16;
+    typedef unsigned __int32 JSUTF32;
+    typedef __int64 JSLONG;
 
-#define EXPORTFUNCTION __declspec(dllexport)
+    #define EXPORTFUNCTION __declspec(dllexport)
 
-#define FASTCALL_MSVC __fastcall
-#define FASTCALL_ATTR
-#define INLINE_PREFIX __inline
+    #define FASTCALL_MSVC __fastcall
+    #define FASTCALL_ATTR
+    #define INLINE_PREFIX __inline
 
 #else
 
-#include <stdint.h>
-typedef int64_t JSINT64;
-typedef uint64_t JSUINT64;
+    #include <stdint.h>
+    typedef int64_t JSINT64;
+    typedef uint64_t JSUINT64;
 
-typedef int32_t JSINT32;
-typedef uint32_t JSUINT32;
+    typedef int32_t JSINT32;
+    typedef uint32_t JSUINT32;
 
-#define FASTCALL_MSVC
+    #define FASTCALL_MSVC
 
-#if !defined __x86_64__
-#define FASTCALL_ATTR __attribute__((fastcall))
-#else
-#define FASTCALL_ATTR
+    #if !defined __x86_64__
+        #define FASTCALL_ATTR __attribute__((fastcall))
+    #else
+        #define FASTCALL_ATTR
+    #endif
+
+    #define INLINE_PREFIX inline
+
+    typedef uint8_t JSUINT8;
+    typedef uint16_t JSUTF16;
+    typedef uint32_t JSUTF32;
+
+    typedef int64_t JSLONG;
+
+    #define EXPORTFUNCTION
 #endif
 
-#define INLINE_PREFIX inline
-
-typedef uint8_t JSUINT8;
-typedef uint16_t JSUTF16;
-typedef uint32_t JSUTF32;
-
-typedef int64_t JSLONG;
-
-#define EXPORTFUNCTION
+#ifdef __GNUC__
+    #define LIKELY(x)       __builtin_expect(!!(x), 1)
+    #define UNLIKELY(x)     __builtin_expect(!!(x), 0)
+#else
+    #define LIKELY(x)       (x)
+    #define UNLIKELY(x)     (x)
 #endif
 
 #if !(defined(__LITTLE_ENDIAN__) || defined(__BIG_ENDIAN__))
 
-#if __BYTE_ORDER__ == __ORDER_LITTLE_ENDIAN__
-#define __LITTLE_ENDIAN__
-#else
+    #if __BYTE_ORDER__ == __ORDER_LITTLE_ENDIAN__
+        #define __LITTLE_ENDIAN__
+    #else
 
-#if __BYTE_ORDER__ == __ORDER_BIG_ENDIAN__
-#define __BIG_ENDIAN__
-#endif
+    #if __BYTE_ORDER__ == __ORDER_BIG_ENDIAN__
+        #define __BIG_ENDIAN__
+    #endif
 
 #endif
 
 #endif
 
 #if !defined(__LITTLE_ENDIAN__) && !defined(__BIG_ENDIAN__)
-#error "Endianess not supported"
+    #error "Endianness not supported"
 #endif
 
 enum JSTYPES
@@ -233,10 +241,6 @@ typedef struct __JSONObjectEncoder
   Configuration for max recursion, set to 0 to use default (see JSON_MAX_RECURSION_DEPTH)*/
   int recursionMax;
 
-  /*
-  Configuration for max decimals of double floating point numbers to encode (0-9) */
-  int doublePrecision;
-
   /*
   If true output will be ASCII with all characters above 127 encoded as \uXXXX. If false output will be UTF-8 or what ever charset strings are brought as */
   int forceASCII;
@@ -297,8 +301,6 @@ JSONObjectEncoder.free or free() as specified when calling this function.
 */
 EXPORTFUNCTION char *JSON_EncodeObject(JSOBJ obj, JSONObjectEncoder *enc, char *buffer, size_t cbBuffer);
 
-
-
 typedef struct __JSONObjectDecoder
 {
   JSOBJ (*newString)(void *prv, wchar_t *start, wchar_t *end);
@@ -319,10 +321,48 @@ typedef struct __JSONObjectDecoder
   JSPFN_REALLOC realloc;
   char *errorStr;
   char *errorOffset;
-  int preciseFloat;
   void *prv;
 } JSONObjectDecoder;
 
 EXPORTFUNCTION JSOBJ JSON_DecodeObject(JSONObjectDecoder *dec, const char *buffer, size_t cbBuffer);
 
+#define DCONV_DECIMAL_IN_SHORTEST_LOW -6
+#define DCONV_DECIMAL_IN_SHORTEST_HIGH 21
+
+enum dconv_d2s_flags {
+  DCONV_D2S_NO_FLAGS = 0,
+  DCONV_D2S_EMIT_POSITIVE_EXPONENT_SIGN = 1,
+  DCONV_D2S_EMIT_TRAILING_DECIMAL_POINT = 2,
+  DCONV_D2S_EMIT_TRAILING_ZERO_AFTER_POINT = 4,
+  DCONV_D2S_UNIQUE_ZERO = 8
+};
+
+enum dconv_s2d_flags
+{
+  DCONV_S2D_NO_FLAGS = 0,
+  DCONV_S2D_ALLOW_HEX = 1,
+  DCONV_S2D_ALLOW_OCTALS = 2,
+  DCONV_S2D_ALLOW_TRAILING_JUNK = 4,
+  DCONV_S2D_ALLOW_LEADING_SPACES = 8,
+  DCONV_S2D_ALLOW_TRAILING_SPACES = 16,
+  DCONV_S2D_ALLOW_SPACES_AFTER_SIGN = 32
+};
+
+void dconv_d2s_init(int flags,
+                    const char* infinity_symbol,
+                    const char* nan_symbol,
+                    char exponent_character,
+                    int decimal_in_shortest_low,
+                    int decimal_in_shortest_high,
+                    int max_leading_padding_zeroes_in_precision_mode,
+                    int max_trailing_padding_zeroes_in_precision_mode);
+int dconv_d2s(double value, char* buf, int buflen, int* strlength);
+void dconv_d2s_free(void);
+
+void dconv_s2d_init(int flags, double empty_string_value,
+                    double junk_string_value, const char* infinity_symbol,
+                    const char* nan_symbol);
+double dconv_s2d(const char* buffer, int length, int* processed_characters_count);
+void dconv_s2d_free(void);
+
 #endif
