@@ -27,11 +27,11 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 
 Portions of code from MODP_ASCII - Ascii transformations (upper/lower, etc)
-http://code.google.com/p/stringencoders/
+https://github.com/client9/stringencoders
 Copyright (c) 2007  Nick Galbreath -- nickg [at] modp [dot] com. All rights reserved.
 
 Numeric decoder derived from from TCL library
-http://www.opensource.apple.com/source/tcl/tcl-14/tcl/license.terms
+https://opensource.apple.com/source/tcl/tcl-14/tcl/license.terms
 * Copyright (c) 1988-1993 The Regents of the University of California.
 * Copyright (c) 1994 Sun Microsystems, Inc.
 */
@@ -44,6 +44,7 @@ http://www.opensource.apple.com/source/tcl/tcl-14/tcl/license.terms
 #include <wchar.h>
 #include <stdlib.h>
 #include <errno.h>
+#include <stdint.h>
 
 #ifndef TRUE
 #define TRUE 1
@@ -66,7 +67,7 @@ struct DecoderState
   JSONObjectDecoder *dec;
 };
 
-JSOBJ FASTCALL_MSVC decode_any( struct DecoderState *ds) FASTCALL_ATTR;
+static JSOBJ FASTCALL_MSVC decode_any( struct DecoderState *ds) FASTCALL_ATTR;
 typedef JSOBJ (*PFN_DECODER)( struct DecoderState *ds);
 
 static JSOBJ SetError( struct DecoderState *ds, int offset, const char *message)
@@ -76,40 +77,22 @@ static JSOBJ SetError( struct DecoderState *ds, int offset, const char *message)
   return NULL;
 }
 
-double createDouble(double intNeg, double intValue, double frcValue, int frcDecimalCount)
+static FASTCALL_ATTR JSOBJ FASTCALL_MSVC decodeDouble(struct DecoderState *ds)
 {
-  static const double g_pow10[] = {1.0, 0.1, 0.01, 0.001, 0.0001, 0.00001, 0.000001,0.0000001, 0.00000001, 0.000000001, 0.0000000001, 0.00000000001, 0.000000000001, 0.0000000000001, 0.00000000000001, 0.000000000000001};
-  return (intValue + (frcValue * g_pow10[frcDecimalCount])) * intNeg;
-}
-
-FASTCALL_ATTR JSOBJ FASTCALL_MSVC decodePreciseFloat(struct DecoderState *ds)
-{
-  char *end;
-  double value;
-  errno = 0;
-
-  value = strtod(ds->start, &end);
-
-  if (errno == ERANGE)
-  {
-    return SetError(ds, -1, "Range error when decoding numeric as double");
-  }
-
-  ds->start = end;
+  int processed_characters_count;
+  int len = (int)(ds->end - ds->start);
+  double value = dconv_s2d(ds->start, len, &processed_characters_count);
+  ds->lastType = JT_DOUBLE;
+  ds->start += processed_characters_count;
   return ds->dec->newDouble(ds->prv, value);
 }
 
-FASTCALL_ATTR JSOBJ FASTCALL_MSVC decode_numeric (struct DecoderState *ds)
+static FASTCALL_ATTR JSOBJ FASTCALL_MSVC decode_numeric (struct DecoderState *ds)
 {
   int intNeg = 1;
-  int mantSize = 0;
   JSUINT64 intValue;
   JSUINT64 prevIntValue;
   int chr;
-  int decimalCount = 0;
-  double frcValue = 0.0;
-  double expNeg;
-  double expValue;
   char *offset = ds->start;
 
   JSUINT64 overflowLimit = LLONG_MAX;
@@ -155,21 +138,18 @@ FASTCALL_ATTR JSOBJ FASTCALL_MSVC decode_numeric (struct DecoderState *ds)
         }
 
         offset ++;
-        mantSize ++;
         break;
       }
       case '.':
       {
         offset ++;
-        goto DECODE_FRACTION;
-        break;
+        return decodeDouble(ds);
       }
       case 'e':
       case 'E':
       {
         offset ++;
-        goto DECODE_EXPONENT;
-        break;
+        return decodeDouble(ds);
       }
 
       default:
@@ -197,119 +177,9 @@ BREAK_INT_LOOP:
   {
     return ds->dec->newInt(ds->prv, (JSINT32) (intValue * intNeg));
   }
-
-DECODE_FRACTION:
-
-  if (ds->dec->preciseFloat)
-  {
-    return decodePreciseFloat(ds);
-  }
-
-  // Scan fraction part
-  frcValue = 0.0;
-  for (;;)
-  {
-    chr = (int) (unsigned char) *(offset);
-
-    switch (chr)
-    {
-      case '0':
-      case '1':
-      case '2':
-      case '3':
-      case '4':
-      case '5':
-      case '6':
-      case '7':
-      case '8':
-      case '9':
-      {
-        if (decimalCount < JSON_DOUBLE_MAX_DECIMALS)
-        {
-          frcValue = frcValue * 10.0 + (double) (chr - 48);
-          decimalCount ++;
-        }
-        offset ++;
-        break;
-      }
-      case 'e':
-      case 'E':
-      {
-        offset ++;
-        goto DECODE_EXPONENT;
-        break;
-      }
-      default:
-      {
-        goto BREAK_FRC_LOOP;
-      }
-    }
-  }
-
-BREAK_FRC_LOOP:
-  //FIXME: Check for arithemtic overflow here
-  ds->lastType = JT_DOUBLE;
-  ds->start = offset;
-  return ds->dec->newDouble (ds->prv, createDouble( (double) intNeg, (double) intValue, frcValue, decimalCount));
-
-DECODE_EXPONENT:
-  if (ds->dec->preciseFloat)
-  {
-    return decodePreciseFloat(ds);
-  }
-
-  expNeg = 1.0;
-
-  if (*(offset) == '-')
-  {
-    expNeg = -1.0;
-    offset ++;
-  }
-  else
-  if (*(offset) == '+')
-  {
-    expNeg = +1.0;
-    offset ++;
-  }
-
-  expValue = 0.0;
-
-  for (;;)
-  {
-    chr = (int) (unsigned char) *(offset);
-
-    switch (chr)
-    {
-      case '0':
-      case '1':
-      case '2':
-      case '3':
-      case '4':
-      case '5':
-      case '6':
-      case '7':
-      case '8':
-      case '9':
-      {
-        expValue = expValue * 10.0 + (double) (chr - 48);
-        offset ++;
-        break;
-      }
-      default:
-      {
-        goto BREAK_EXP_LOOP;
-      }
-    }
-  }
-
-BREAK_EXP_LOOP:
-  //FIXME: Check for arithemtic overflow here
-  ds->lastType = JT_DOUBLE;
-  ds->start = offset;
-  return ds->dec->newDouble (ds->prv, createDouble( (double) intNeg, (double) intValue , frcValue, decimalCount) * pow(10.0, expValue * expNeg));
 }
 
-FASTCALL_ATTR JSOBJ FASTCALL_MSVC decode_true ( struct DecoderState *ds)
+static FASTCALL_ATTR JSOBJ FASTCALL_MSVC decode_true ( struct DecoderState *ds)
 {
   char *offset = ds->start;
   offset ++;
@@ -329,7 +199,7 @@ SETERROR:
   return SetError(ds, -1, "Unexpected character found when decoding 'true'");
 }
 
-FASTCALL_ATTR JSOBJ FASTCALL_MSVC decode_false ( struct DecoderState *ds)
+static FASTCALL_ATTR JSOBJ FASTCALL_MSVC decode_false ( struct DecoderState *ds)
 {
   char *offset = ds->start;
   offset ++;
@@ -351,7 +221,7 @@ SETERROR:
   return SetError(ds, -1, "Unexpected character found when decoding 'false'");
 }
 
-FASTCALL_ATTR JSOBJ FASTCALL_MSVC decode_null ( struct DecoderState *ds)
+static FASTCALL_ATTR JSOBJ FASTCALL_MSVC decode_null ( struct DecoderState *ds)
 {
   char *offset = ds->start;
   offset ++;
@@ -371,7 +241,7 @@ SETERROR:
   return SetError(ds, -1, "Unexpected character found when decoding 'null'");
 }
 
-FASTCALL_ATTR void FASTCALL_MSVC SkipWhitespace(struct DecoderState *ds)
+static FASTCALL_ATTR void FASTCALL_MSVC SkipWhitespace(struct DecoderState *ds)
 {
   char *offset = ds->start;
 
@@ -422,7 +292,7 @@ static const JSUINT8 g_decoderLookup[256] =
   /* 0xf0 */ 4, 4, 4, 4, 4, 4, 4, 4, DS_UTFLENERROR, DS_UTFLENERROR, DS_UTFLENERROR, DS_UTFLENERROR, DS_UTFLENERROR, DS_UTFLENERROR, DS_UTFLENERROR, DS_UTFLENERROR,
 };
 
-FASTCALL_ATTR JSOBJ FASTCALL_MSVC decode_string ( struct DecoderState *ds)
+static FASTCALL_ATTR JSOBJ FASTCALL_MSVC decode_string ( struct DecoderState *ds)
 {
   JSUTF16 sur[2] = { 0 };
   int iSur = 0;
@@ -496,6 +366,7 @@ FASTCALL_ATTR JSOBJ FASTCALL_MSVC decode_string ( struct DecoderState *ds)
         return SetError (ds, -1, "Invalid UTF-8 sequence length when decoding 'string'");
       }
       case DS_ISESCAPE:
+      {
         inputOffset ++;
         switch (*inputOffset)
         {
@@ -581,13 +452,14 @@ FASTCALL_ATTR JSOBJ FASTCALL_MSVC decode_string ( struct DecoderState *ds)
 #endif
               iSur = 0;
             }
-          break;
-        }
+            break;
+          }
 
-        case '\0': return SetError(ds, -1, "Unterminated escape sequence when decoding 'string'");
-        default: return SetError(ds, -1, "Unrecognized escape sequence when decoding 'string'");
+          case '\0': return SetError(ds, -1, "Unterminated escape sequence when decoding 'string'");
+          default: return SetError(ds, -1, "Unrecognized escape sequence when decoding 'string'");
+        }
+        break;
       }
-      break;
 
       case 1:
       {
@@ -672,7 +544,7 @@ FASTCALL_ATTR JSOBJ FASTCALL_MSVC decode_string ( struct DecoderState *ds)
   }
 }
 
-FASTCALL_ATTR JSOBJ FASTCALL_MSVC decode_array(struct DecoderState *ds)
+static FASTCALL_ATTR JSOBJ FASTCALL_MSVC decode_array(struct DecoderState *ds)
 {
   JSOBJ itemValue;
   JSOBJ newObj;
@@ -732,22 +604,25 @@ FASTCALL_ATTR JSOBJ FASTCALL_MSVC decode_array(struct DecoderState *ds)
       return SetError(ds, -1, "Unexpected character found when decoding array value (2)");
     }
 
-    len ++;
+    len++;
   }
 }
 
-FASTCALL_ATTR JSOBJ FASTCALL_MSVC decode_object( struct DecoderState *ds)
+static FASTCALL_ATTR JSOBJ FASTCALL_MSVC decode_object( struct DecoderState *ds)
 {
   JSOBJ itemName;
   JSOBJ itemValue;
   JSOBJ newObj;
+  int len;
 
   ds->objDepth++;
-  if (ds->objDepth > JSON_MAX_OBJECT_DEPTH) {
+  if (ds->objDepth > JSON_MAX_OBJECT_DEPTH)
+  {
     return SetError(ds, -1, "Reached object decoding depth limit");
   }
 
   newObj = ds->dec->newObject(ds->prv);
+  len = 0;
 
   ds->start ++;
 
@@ -758,8 +633,14 @@ FASTCALL_ATTR JSOBJ FASTCALL_MSVC decode_object( struct DecoderState *ds)
     if ((*ds->start) == '}')
     {
       ds->objDepth--;
-      ds->start ++;
-      return newObj;
+      if (len == 0)
+      {
+        ds->start ++;
+        return newObj;
+      }
+
+      ds->dec->releaseObject(ds->prv, newObj);
+      return SetError(ds, -1, "Unexpected character in found when decoding object value");
     }
 
     ds->lastType = JT_INVALID;
@@ -816,10 +697,12 @@ FASTCALL_ATTR JSOBJ FASTCALL_MSVC decode_object( struct DecoderState *ds)
         ds->dec->releaseObject(ds->prv, newObj);
         return SetError(ds, -1, "Unexpected character in found when decoding object value");
     }
+
+    len++;
   }
 }
 
-FASTCALL_ATTR JSOBJ FASTCALL_MSVC decode_any(struct DecoderState *ds)
+static FASTCALL_ATTR JSOBJ FASTCALL_MSVC decode_any(struct DecoderState *ds)
 {
   for (;;)
   {
